@@ -1,11 +1,14 @@
-# $Id: XPath.pm,v 1.27 2000/04/17 17:07:41 matt Exp $
+# $Id: XPath.pm,v 1.29 2000/04/22 19:34:18 matt Exp $
 
 package XML::XPath;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD $revision);
 
-$VERSION = '0.22';
+$VERSION = '0.23';
+
+$XML::XPath::Namespaces = 1;
+$XML::XPath::Debug = 0;
 
 use XML::XPath::XMLParser;
 use XML::XPath::Parser;
@@ -62,6 +65,33 @@ sub find {
 	return $parsed_path->evaluate($context);
 }
 
+sub findnodes {
+	my $self = shift;
+	my ($path, $context) = @_;
+	
+	my $results = $self->find($path, $context);
+	
+	if ($results->isa('XML::XPath::NodeSet')) {
+		return $results->get_nodelist;
+	}
+	
+	warn("findnodes returned a ", ref($results), " object\n") if $XML::XPath::Debug;
+	return XML::XPath::NodeSet->new();
+}
+
+sub findvalue {
+	my $self = shift;
+	my ($path, $context) = @_;
+	
+	my $results = $self->find($path, $context);
+	
+	if ($results->isa('XML::XPath::NodeSet')) {
+		return $results->to_literal;
+	}
+	
+	return $results;
+}
+
 sub cleanup {
 	my $self = shift;
 	my $context = $self->get_context;
@@ -147,7 +177,7 @@ this as they support functionality beyond XPath.
 			XML::XPath::XMLParser::as_string($node),
 			"\n\n";
 	}
-	
+
 =head1 DETAILS
 
 There's an awful lot to all of this, so bear with it - if you stick it
@@ -186,7 +216,7 @@ the life of your application. This is because the object (and it's sub-objects)
 maintain certain bits of state information that will be useful (such
 as XPath variables) to later calls to find(). It's also a good idea because
 you'll use less memory this way.
-	
+
 =head2 I<nodeset> = find($path, [$context])
 
 The find function takes an XPath expression (a string) and returns either an
@@ -199,6 +229,26 @@ See L<XML::XPath::NodeSet>. An optional second parameter of a context
 node allows you to use this method repeatedly, for example XSLT needs
 to do this.
 
+=head2 findnodes($path, [$context])
+
+Returns a list of nodes found by $path, optionally in context $context.
+
+=head2 findvalue($path, [$context])
+
+Returns either a C<XML::XPath::Literal>, a C<XML::XPath::Boolean> or a
+C<XML::XPath::Number> object. If the path returns a NodeSet,
+$nodeset->to_literal is called automatically for you (and thus a
+C<XML::XPath::Literal is returned). Note that
+for each of the objects stringification is overloaded, so you can just
+print the value found, or manipulate it in the ways you would a normal
+perl value (e.g. using regular expressions).
+
+=head2 $XML::XPath::Namespaces
+
+Set this to 0 if you I<don't> want namespace processing to occur. This
+will make everything a little (tiny) bit faster, but you'll suffer for it,
+probably.
+
 =head1 IMPORTANT
 
 The node format used by XML::XPath contains circular references. This
@@ -209,36 +259,40 @@ of trouble!). An example would be if you have a long-running process
 (e.g. mod_perl) that uses this module. If you just did the following
 (this is mod_perl specific, but you should get the idea):
 
-	sub handler {
-		my $r = shift;
-		my $xp = XML::XPath->new( filename => $r->filename );
-		
-		my $nodes = $xp->find("//h1");
-		
-		foreach my $node ($nodes->get_nodelist) {
-			print XML::XPath::XMLParser::as_string($node), "\n\n";
-		}
-	}
+  sub handler {
+    my $r = shift;
+    my $xp = XML::XPath->new( filename => $r->filename );
+  
+    my $nodes = $xp->find("//h1");
+  
+    foreach my $node ($nodes->get_nodelist) {
+      print XML::XPath::XMLParser::as_string($node), "\n\n";
+    }
+  }
 
 You would find your process size growing and growing. You have to
 manually delete those circular references. It's not all bad though -
 I've provided you with a cleanup method that you can use:
 
-	sub handler {
-		my $r = shift;
-		my $xp = XML::XPath->new( filename => $r->filename );
-		
-		my $nodes = $xp->find("//h1");
-		
-		foreach my $node ($nodes->get_nodelist) {
-			print XML::XPath::XMLParser::as_string($node), "\n\n";
-		}
-		$xp->cleanup();
-	}
+  sub handler {
+    my $r = shift;
+    my $xp = XML::XPath->new( filename => $r->filename );
+  
+    my $nodes = $xp->find("//h1");
+  
+    foreach my $node ($nodes->get_nodelist) {
+      print XML::XPath::XMLParser::as_string($node), "\n\n";
+    }
+    $xp->cleanup();
+  }
 
 Beware that nodes are completely useless after they've been disposed
 of.
 
+=head1 Example
+
+There are some complete XPath examples on http://xml.sergeant.org/xpath.xml
+
 =head1 Support/Author
 
 This module is copyright 2000 Fastnet Software Ltd. This is free
